@@ -318,8 +318,6 @@ class Node(AbstractNode):
                 rpc = BitcoinRPC(
                     **rpc_conf_arr[0], proxy_url=self.proxy_url, only_tor=self.only_tor
                 )
-            if rpc == None:
-                logger.warning(f"No rpc was found for {self}")
             return rpc
         else:
             # if autodetect is disabled and port is not defined
@@ -466,19 +464,6 @@ class Node(AbstractNode):
                 logger.debug(
                     f"connection {self.rpc} failed test_connection in check_info:"
                 )
-                try:
-                    self.rpc.multi(
-                        [
-                            ("getblockchaininfo", None),
-                            ("getnetworkinfo", None),
-                            ("getmempoolinfo", None),
-                            ("uptime", None),
-                            ("getblockhash", 0),
-                            ("scantxoutset", "status", []),
-                        ]
-                    )
-                except Exception as e:
-                    logger.exception(e)
             self._mark_node_as_broken()
 
     def test_rpc(self):
@@ -638,7 +623,8 @@ class Node(AbstractNode):
     def network_parameters(self):
         try:
             return self._network_parameters
-        except Exception:
+        except Exception as e:
+            logger.exception(e)
             return get_network("main")
 
     @property
@@ -660,6 +646,7 @@ class Node(AbstractNode):
                 self.info.get("softforks", {}).get("taproot", {}).get("active", False)
             )
         except Exception as e:
+            logger.exception(e)
             return False
 
     @property
