@@ -1,5 +1,6 @@
 import random
 import traceback
+import logging
 from pathlib import Path
 from time import time
 
@@ -10,7 +11,11 @@ from werkzeug.exceptions import MethodNotAllowed, NotFound
 
 from ..server_endpoints import flash
 from ..services.callbacks import flask_before_request
-from ..specter_error import ExtProcTimeoutException, SpecterError
+from ..specter_error import (
+    ExtProcTimeoutException,
+    SpecterError,
+    BrokenCoreConnectionException,
+)
 
 env_path = Path(".") / ".flaskenv"
 from dotenv import load_dotenv
@@ -36,7 +41,7 @@ from .price import price_endpoint
 from .settings import settings_endpoint
 from .setup import setup_endpoint
 from .wallets import wallets_endpoint
-from .wallets_api import wallets_endpoint_api
+from .wallets.wallets_api import wallets_endpoint_api
 from .welcome import welcome_endpoint
 
 spc_prefix = app.config["SPECTER_URL_PREFIX"]
@@ -52,17 +57,22 @@ app.register_blueprint(wallets_endpoint, url_prefix=f"{spc_prefix}/wallets")
 app.register_blueprint(wallets_endpoint_api, url_prefix=f"{spc_prefix}/wallets")
 
 rand = random.randint(0, 1e32)  # to force style refresh
+logger = logging.getLogger(__name__)
 
 ########## exception handlers ##############
 @app.errorhandler(RpcError)
 def server_rpc_error(rpce):
     """Specific SpecterErrors get passed on to the User as flash"""
+    if request.headers.get("Accept") == "application/json":
+        return {"error": str(rpce)}
     if rpce.error_code == -18:  # RPC_WALLET_NOT_FOUND
+
         flash(
             _("Wallet not found. Specter reloaded all wallets, please try again."),
             "error",
         )
     else:
+
         flash(_("Bitcoin Core RpcError: {}").format(str(rpce)), "error")
     try:
         app.specter.wallet_manager.update()
@@ -73,7 +83,9 @@ def server_rpc_error(rpce):
 
 @app.errorhandler(SpecterError)
 def server_specter_error(se):
-    """Specific SpecterErrors get passed on to the User as flash"""
+    """Specific SpecterErrors get passed on to the User as flash (or in error-field for json)"""
+    if request.headers.get("Accept") == "application/json":
+        return {"error": str(se)}
     flash(str(se), "error")
     try:
         app.specter.wallet_manager.update()
@@ -90,40 +102,56 @@ def server_specter_error(se):
 def server_notFound_error(e):
     """Unspecific Exceptions get a 404 Error-Page"""
     # if rpc is not available
-    app.logger.error("Could not find Resource (404): %s" % request.url)
+    error_msg = "Could not find Resource (404): %s" % request.url
+    app.logger.error(error_msg)
+    if request.headers.get("Accept") == "application/json":
+        return {"error": error_msg}
     return render_template("500.jinja", error=e), 404
 
 
 @app.errorhandler(Exception)
 def server_error(e):
     """Unspecific Exceptions get a 500 Error-Page"""
-    # if rpc is not available
-    if app.specter.rpc is None or not app.specter.rpc.test_connection():
-        # make sure specter knows that rpc is not there
-        app.specter.check()
-    app.logger.error("Uncaught exception: %s" % e)
+    error_msg = "Uncaught exception: %s" % e
+    app.logger.error(error_msg)
     trace = traceback.format_exc()
     app.logger.error(trace)
+    if request.headers.get("Accept") == "application/json":
+        return {"error": error_msg}
     return render_template("500.jinja", error=e, traceback=trace), 500
 
 
+@app.errorhandler(BrokenCoreConnectionException)
+def server_broken_core_connection(e):
+    error_msg = "You got disconnected from your node (no RPC connection)"
+    logger.exception(e)
+    if request.headers.get("Accept") == "application/json":
+        return {"error": error_msg}
+    flash(error_msg, "error")
+    try:
+        app.specter.check()
+        return redirect(url_for("welcome_endpoint.about"))
+    except Exception as e:
+        logger.exception(e)
+        return server_error(e)
+
+
 @app.errorhandler(ExtProcTimeoutException)
 def server_error_timeout(e):
     """Unspecific Exceptions get a 500 Error-Page"""
-    # if rpc is not available
+    error_msg = _(
+        "Bitcoin Core is not coming up in time. Maybe it's just slow but please check the logs below"
+    )
     if app.specter.rpc is None or not app.specter.rpc.test_connection():
         # make sure specter knows that rpc is not there
         app.specter.check()
     app.logger.error("ExternalProcessTimeoutException: %s" % e)
-    flash(
-        _(
-            "Bitcoin Core is not coming up in time. Maybe it's just slow but please check the logs below"
-        ),
-        "warn",
-    )
+    if request.headers.get("Accept") == "application/json":
+        return {"error": error_msg}
+    flash(error_msg, "warn")
     return redirect(
         url_for(
-            "node_settings.bitcoin_core_internal_logs",
+            "nodes_endpoint.internal_node_logs",
             node_alias=app.specter.node.alias,
         )
     )
@@ -134,19 +162,25 @@ def server_error_csrf(e):
     """CSRF token missing. Most likely session expired.
     If persisting after refresh this could mean the front-end
     is not sending the CSRF token properly in some form"""
+    error_msg = _("Session expired (CSRF). Please refresh and try again.")
     app.logger.error("CSRF Exception: %s" % e)
     trace = traceback.format_exc()
     app.logger.error(trace)
-    flash(_("Session expired. Please refresh and try again."), "error")
+    if request.headers.get("Accept") == "application/json":
+        return {"error": error_msg}
+    flash(error_msg, "error")
     return redirect(request.url)
 
 
 @app.errorhandler(MethodNotAllowed)
 def server_error_405(e):
     """405 method not allowed. Token might have expired."""
-    app.logger.error("405 MethodNotAllowed Exception: %s" % e)
+    error_msg = _("405 method not allowed. Token might have expired.")
+    app.logger.error(f"{error_msg} : {e}")
     trace = traceback.format_exc()
     app.logger.error(trace)
+    if request.headers.get("Accept") == "application/json":
+        return {"error": error_msg}
     flash(_("Session expired. Please refresh and try again."), "error")
     return redirect(request.url)
 
@@ -157,8 +191,6 @@ def selfcheck():
     """check status before every request"""
     if app.specter.rpc is not None:
         type(app.specter.rpc).counter = 0
-        if not app.specter.chain:
-            app.specter.check()
     if app.config.get("LOGIN_DISABLED"):
         app.login("admin")
 
@@ -179,7 +211,7 @@ def slow_request_detection_stop(response):
     try:
         diff = time() - g.start
     except Exception as e:
-        app.logger.error(e)
+        app.logger.exception(e)
         return response
     if (
         (response.response)
